import { Injectable, NgModule } from '@angular/core';
import { HttpResponse, HttpErrorResponse, HttpParams, HTTP_INTERCEPTORS } from '@angular/common/http';
import { throwError, of } from 'rxjs';
import { timeout, map, catchError, take } from 'rxjs/operators';
import { CommonsType } from 'tscommons-core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const API_TIMEOUT = 30 * 1000;
// 30 seconds
class AdamantineApiInterceptor {
    constructor() { }
    /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    intercept(req, next) {
        return next.handle(req)
            .pipe(timeout(API_TIMEOUT), map((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            if (event instanceof HttpResponse && CommonsType.isObject(event.body) && CommonsType.hasProperty(event.body, 'outcome')) {
                // assume API response
                if (event.body.outcome === 'error') {
                    return throwError(event.body.message);
                }
                if (event.body.outcome === 'success') {
                    return event.clone({ body: event.body.result });
                }
                return throwError('Unknown outcome');
            }
            // assume not an API response
            return event;
        })), catchError((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            if (event instanceof HttpErrorResponse) {
                try {
                    /** @type {?} */
                    const json = JSON.parse(event.error);
                    if (-1 !== Object.keys(json).indexOf('outcome')) {
                        // assume API response
                        if (json.outcome === 'error') {
                            return throwError(json.message);
                        }
                        return throwError('Unknown outcome');
                    }
                    // assume not an API response
                    return throwError(event);
                }
                catch (e) {
                    return throwError(event);
                }
            }
            return throwError(event);
        })));
    }
}
AdamantineApiInterceptor.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AdamantineApiInterceptor.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class AdamantineApiService {
    /**
     * @param {?} rootURL
     * @param {?} http
     * @param {?=} defaultMaxCacheAge
     */
    constructor(rootURL, http, defaultMaxCacheAge) {
        this.rootURL = rootURL;
        this.http = http;
        this.defaultMaxCacheAge = defaultMaxCacheAge;
        this.cache = new Map();
    }
    /**
     * @private
     * @param {?=} params
     * @return {?}
     */
    static buildParams(params) {
        /** @type {?} */
        let hp = new HttpParams();
        if (params) {
            for (const key of Object.keys(params)) {
                hp = hp.set(key, params[key]);
            }
        }
        return hp;
    }
    /**
     * @protected
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxCacheAge
     * @return {?}
     */
    internalGetObservable(script, params, maxCacheAge) {
        /** @type {?} */
        const cacheIndex = JSON.stringify({
            script: script,
            params: params
        });
        maxCacheAge = maxCacheAge || this.defaultMaxCacheAge;
        if (maxCacheAge && this.cache.has(cacheIndex)) {
            /** @type {?} */
            const threshold = new Date();
            threshold.setMilliseconds(-maxCacheAge);
            if (this.cache.has(cacheIndex)) {
                /** @type {?} */
                const entry = this.cache.get(cacheIndex);
                if (entry) {
                    if (entry.age > threshold) {
                        /** @type {?} */
                        const json = JSON.parse(entry.data);
                        return of(json);
                    }
                    // expired
                    this.cache.delete(cacheIndex);
                }
            }
        }
        return this.http
            .get(`${this.rootURL}${script}`, { params: AdamantineApiService.buildParams(params) })
            .pipe(map((/**
         * @param {?} result
         * @return {?}
         */
        (result) => {
            if (maxCacheAge) {
                this.cache.set(cacheIndex, {
                    age: new Date(),
                    data: JSON.stringify(result)
                });
            }
            return result;
        })), take(1));
    }
    /**
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxCacheAge
     * @return {?}
     */
    getObservable(script, params, maxCacheAge) {
        return this.internalGetObservable(script, params, maxCacheAge);
    }
    /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    internalPostObservable(script, params) {
        return this.http
            .post(`${this.rootURL}${script}`, params)
            .pipe(take(1));
    }
    /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    postObservable(script, params) {
        return this.internalPostObservable(script, params);
    }
    /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    internalPutObservable(script, params) {
        return this.http
            .put(`${this.rootURL}${script}`, params)
            .pipe(take(1));
    }
    /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    putObservable(script, params) {
        return this.internalPutObservable(script, params);
    }
    /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    internalPatchObservable(script, params) {
        return this.http
            .patch(`${this.rootURL}${script}`, params)
            .pipe(take(1));
    }
    /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    patchObservable(script, params) {
        return this.internalPatchObservable(script, params);
    }
    /**
     * @protected
     * @param {?} script
     * @return {?}
     */
    internalDeleteObservable(script) {
        return this.http
            .delete(`${this.rootURL}${script}`)
            .pipe(take(1));
    }
    /**
     * @param {?} script
     * @return {?}
     */
    deleteObservable(script) {
        return this.internalDeleteObservable(script);
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    AdamantineApiService.prototype.cache;
    /**
     * @type {?}
     * @private
     */
    AdamantineApiService.prototype.rootURL;
    /**
     * @type {?}
     * @private
     */
    AdamantineApiService.prototype.http;
    /**
     * @type {?}
     * @private
     */
    AdamantineApiService.prototype.defaultMaxCacheAge;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// NB, we can't use a forRoot() here, as the interceptors don't get loaded for sub-modules.
class NgxAdamantineCommonsApiModule {
}
NgxAdamantineCommonsApiModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                providers: [
                    {
                        provide: HTTP_INTERCEPTORS,
                        useClass: AdamantineApiInterceptor,
                        multi: true
                    }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AdamantineApiInterceptor, AdamantineApiService, NgxAdamantineCommonsApiModule };
//# sourceMappingURL=ngx-adamantinecommons-api.js.map
