import { Injectable, NgModule } from '@angular/core';
import { HttpResponse, HttpErrorResponse, HttpParams, HTTP_INTERCEPTORS } from '@angular/common/http';
import { throwError, of } from 'rxjs';
import { timeout, map, catchError, take } from 'rxjs/operators';
import { CommonsType } from 'tscommons-core';
import { __values } from 'tslib';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var API_TIMEOUT = 30 * 1000;
// 30 seconds
var AdamantineApiInterceptor = /** @class */ (function () {
    function AdamantineApiInterceptor() {
    }
    /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    AdamantineApiInterceptor.prototype.intercept = /**
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    function (req, next) {
        return next.handle(req)
            .pipe(timeout(API_TIMEOUT), map((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            if (event instanceof HttpResponse && CommonsType.isObject(event.body) && CommonsType.hasProperty(event.body, 'outcome')) {
                // assume API response
                if (event.body.outcome === 'error') {
                    return throwError(event.body.message);
                }
                if (event.body.outcome === 'success') {
                    return event.clone({ body: event.body.result });
                }
                return throwError('Unknown outcome');
            }
            // assume not an API response
            return event;
        })), catchError((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            if (event instanceof HttpErrorResponse) {
                try {
                    /** @type {?} */
                    var json = JSON.parse(event.error);
                    if (-1 !== Object.keys(json).indexOf('outcome')) {
                        // assume API response
                        if (json.outcome === 'error') {
                            return throwError(json.message);
                        }
                        return throwError('Unknown outcome');
                    }
                    // assume not an API response
                    return throwError(event);
                }
                catch (e) {
                    return throwError(event);
                }
            }
            return throwError(event);
        })));
    };
    AdamantineApiInterceptor.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    AdamantineApiInterceptor.ctorParameters = function () { return []; };
    return AdamantineApiInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
var  /**
 * @abstract
 */
AdamantineApiService = /** @class */ (function () {
    function AdamantineApiService(rootURL, http, defaultMaxCacheAge) {
        this.rootURL = rootURL;
        this.http = http;
        this.defaultMaxCacheAge = defaultMaxCacheAge;
        this.cache = new Map();
    }
    /**
     * @private
     * @param {?=} params
     * @return {?}
     */
    AdamantineApiService.buildParams = /**
     * @private
     * @param {?=} params
     * @return {?}
     */
    function (params) {
        var e_1, _a;
        /** @type {?} */
        var hp = new HttpParams();
        if (params) {
            try {
                for (var _b = __values(Object.keys(params)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var key = _c.value;
                    hp = hp.set(key, params[key]);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        return hp;
    };
    /**
     * @protected
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxCacheAge
     * @return {?}
     */
    AdamantineApiService.prototype.internalGetObservable = /**
     * @protected
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxCacheAge
     * @return {?}
     */
    function (script, params, maxCacheAge) {
        var _this = this;
        /** @type {?} */
        var cacheIndex = JSON.stringify({
            script: script,
            params: params
        });
        maxCacheAge = maxCacheAge || this.defaultMaxCacheAge;
        if (maxCacheAge && this.cache.has(cacheIndex)) {
            /** @type {?} */
            var threshold = new Date();
            threshold.setMilliseconds(-maxCacheAge);
            if (this.cache.has(cacheIndex)) {
                /** @type {?} */
                var entry = this.cache.get(cacheIndex);
                if (entry) {
                    if (entry.age > threshold) {
                        /** @type {?} */
                        var json = JSON.parse(entry.data);
                        return of(json);
                    }
                    // expired
                    this.cache.delete(cacheIndex);
                }
            }
        }
        return this.http
            .get("" + this.rootURL + script, { params: AdamantineApiService.buildParams(params) })
            .pipe(map((/**
         * @param {?} result
         * @return {?}
         */
        function (result) {
            if (maxCacheAge) {
                _this.cache.set(cacheIndex, {
                    age: new Date(),
                    data: JSON.stringify(result)
                });
            }
            return result;
        })), take(1));
    };
    /**
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxCacheAge
     * @return {?}
     */
    AdamantineApiService.prototype.getObservable = /**
     * @param {?} script
     * @param {?=} params
     * @param {?=} maxCacheAge
     * @return {?}
     */
    function (script, params, maxCacheAge) {
        return this.internalGetObservable(script, params, maxCacheAge);
    };
    /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    AdamantineApiService.prototype.internalPostObservable = /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    function (script, params) {
        return this.http
            .post("" + this.rootURL + script, params)
            .pipe(take(1));
    };
    /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    AdamantineApiService.prototype.postObservable = /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    function (script, params) {
        return this.internalPostObservable(script, params);
    };
    /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    AdamantineApiService.prototype.internalPutObservable = /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    function (script, params) {
        return this.http
            .put("" + this.rootURL + script, params)
            .pipe(take(1));
    };
    /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    AdamantineApiService.prototype.putObservable = /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    function (script, params) {
        return this.internalPutObservable(script, params);
    };
    /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    AdamantineApiService.prototype.internalPatchObservable = /**
     * @protected
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    function (script, params) {
        return this.http
            .patch("" + this.rootURL + script, params)
            .pipe(take(1));
    };
    /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    AdamantineApiService.prototype.patchObservable = /**
     * @param {?} script
     * @param {?} params
     * @return {?}
     */
    function (script, params) {
        return this.internalPatchObservable(script, params);
    };
    /**
     * @protected
     * @param {?} script
     * @return {?}
     */
    AdamantineApiService.prototype.internalDeleteObservable = /**
     * @protected
     * @param {?} script
     * @return {?}
     */
    function (script) {
        return this.http
            .delete("" + this.rootURL + script)
            .pipe(take(1));
    };
    /**
     * @param {?} script
     * @return {?}
     */
    AdamantineApiService.prototype.deleteObservable = /**
     * @param {?} script
     * @return {?}
     */
    function (script) {
        return this.internalDeleteObservable(script);
    };
    return AdamantineApiService;
}());
if (false) {
    /**
     * @type {?}
     * @private
     */
    AdamantineApiService.prototype.cache;
    /**
     * @type {?}
     * @private
     */
    AdamantineApiService.prototype.rootURL;
    /**
     * @type {?}
     * @private
     */
    AdamantineApiService.prototype.http;
    /**
     * @type {?}
     * @private
     */
    AdamantineApiService.prototype.defaultMaxCacheAge;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// NB, we can't use a forRoot() here, as the interceptors don't get loaded for sub-modules.
var NgxAdamantineCommonsApiModule = /** @class */ (function () {
    function NgxAdamantineCommonsApiModule() {
    }
    NgxAdamantineCommonsApiModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule
                    ],
                    providers: [
                        {
                            provide: HTTP_INTERCEPTORS,
                            useClass: AdamantineApiInterceptor,
                            multi: true
                        }
                    ]
                },] }
    ];
    return NgxAdamantineCommonsApiModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AdamantineApiInterceptor, AdamantineApiService, NgxAdamantineCommonsApiModule };
//# sourceMappingURL=ngx-adamantinecommons-api.js.map
